package be.maximvdw.ac;

import java.util.TreeMap;

import org.bukkit.ChatColor;
import org.bukkit.OfflinePlayer;
import org.bukkit.entity.Player;
import org.kitteh.tag.TagAPI;

import be.maximvdw.ac.tab.TabAPI;

/**
 * ACPlayer
 * 
 * Holds the player information such as specific data and pvp.
 * 
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class ACPlayer {
	private OfflinePlayer player = null; // Player
	private ACStatus defaultStatus = ACStatus.WHITE;
	private ACStatus globalStatus = ACStatus.WHITE; // Normal status
	private boolean pvpGlobal = false; // Allow pvp
	private boolean globalChat = true; // Global chat on
	private boolean loginReset = false; // Reset on relog
	private Channel chatChannel = Channel.GLOBAL; // Chat channel
	private Player lastPM = null; // Last private message
	// Player specific colors
	private TreeMap<String, ACStatus> playerStatus = new TreeMap<String, ACStatus>();
	// Pvp indicator
	private TreeMap<String, Boolean> pvp = new TreeMap<String, Boolean>();

	/**
	 * Create a chat player
	 * 
	 * @param player
	 *            Player
	 */
	public ACPlayer(OfflinePlayer player) {
		this.player = player;
	}

	/**
	 * Get default status
	 * 
	 * @return Default status
	 */
	public ACStatus getDefaultStatus() {
		return defaultStatus;
	}

	/**
	 * Set default status
	 * 
	 * @param status
	 *            Status
	 */
	public void setDefaultStatus(ACStatus status) {
		this.defaultStatus = status;
	}

	/**
	 * Set reset on login
	 */
	public void setResetOnLogin() {
		this.loginReset = true;
	}

	/**
	 * Get the chat channel
	 * 
	 * @return Channel
	 */
	public Channel getChatChannel() {
		return chatChannel;
	}

	/**
	 * Set the chat channel
	 * 
	 * @param channel
	 *            Channel
	 */
	public void setChatChannel(Channel channel) {
		this.chatChannel = channel;
	}

	/**
	 * Set keep on login
	 */
	public void setKeepOnLogin() {
		this.loginReset = false;
	}

	/**
	 * Get the login reset
	 * 
	 * @return Login reset
	 */
	public boolean getLoginReset() {
		return loginReset;
	}

	/**
	 * Get Player specific statuses
	 * 
	 * @return ACStatus treemap
	 */
	public TreeMap<String, ACStatus> getPlayerSpecificStatuses() {
		return playerStatus;
	}

	/**
	 * Get Player specific pvp state
	 * 
	 * @return PvP treemap
	 */
	public TreeMap<String, Boolean> getPlayerSpeificPvp() {
		return pvp;
	}

	/**
	 * Get Player
	 * 
	 * @return Player
	 */
	public Player getPlayer() {
		return player.getPlayer();
	}

	/**
	 * Get the last private messenger
	 * 
	 * @return Player
	 */
	public Player getLastMessenger() {
		return lastPM;
	}

	/**
	 * Set the last private messenger
	 * 
	 * @param player
	 *            Player
	 */
	public void setLastMessage(Player player) {
		this.lastPM = player;
	}

	/**
	 * Get last PM Player
	 * 
	 * @return Player
	 */
	public Player getLastPMPlayer() {
		return lastPM;
	}

	/**
	 * Set last PM Player
	 * 
	 * @param player
	 *            Player
	 */
	public void setLastPMPlayer(Player player) {
		this.lastPM = player;
	}

	/**
	 * Set the player
	 * 
	 * @param player
	 *            Player
	 */
	public void setPlayer(Player player) {
		this.player = player;
	}

	/**
	 * Set the global status
	 * 
	 * @param status
	 *            ACStatus
	 */
	public void setGlobalStatus(ACStatus status) {
		this.globalStatus = status;
		if (this.player.getPlayer() != null) {
			TagAPI.refreshPlayer(this.player.getPlayer());
			TabAPI.updateTablists();
		}
	}

	/**
	 * Set global chat raw
	 * 
	 * @param state
	 *            State
	 */
	public void setGlobalChatRaw(boolean state) {
		globalChat = state;
	}

	/**
	 * Check if player wants global chat
	 * 
	 * @return Boolean
	 */
	public boolean getGlobalChat() {
		return globalChat;
	}

	/**
	 * Set global chat state
	 * 
	 * @param chat
	 *            Chat
	 */
	public void setGlobalChat(boolean chat) {
		this.globalChat = chat;
		if (this.player.getPlayer() != null) {
			TagAPI.refreshPlayer(this.player.getPlayer());
			TabAPI.updateTablists();
		}
	}

	/**
	 * Get the global status
	 * 
	 * @return ACStatus
	 */
	public ACStatus getGlobalStatus() {
		return globalStatus;
	}

	/**
	 * Set specfic status
	 * 
	 * @param status
	 *            AC Status
	 * @param player
	 *            Player
	 */
	public void setStatus(ACStatus status, OfflinePlayer player) {
		playerStatus.put(player.getName().toLowerCase(), status);
		if (player.getPlayer() != null && player.getPlayer() != null) {
			TagAPI.refreshPlayer(getPlayer(), player.getPlayer());
			TabAPI.updateTablist(player.getPlayer());
		}
	}

	/**
	 * Get specific status
	 * 
	 * @param target
	 *            Player
	 * @return AC Status
	 */
	public ACStatus getStatus(OfflinePlayer target) {
		if (target != null) {
			if (playerStatus.containsKey((target.getName().toLowerCase()))) {
				return playerStatus.get(target.getName().toLowerCase());
			}
		}
		return getGlobalStatus();
	}

	/**
	 * Allow PvP with a specific player
	 * 
	 * @param player
	 *            Player
	 */
	public void allowPvp(OfflinePlayer player) {
		pvp.put(player.getName().toLowerCase(), true);
		if (player.getPlayer() != null && this.player.getPlayer() != null) {
			TagAPI.refreshPlayer(this.player.getPlayer(), player.getPlayer());
			TabAPI.updateTablist(player.getPlayer());
		}
	}

	/**
	 * Deny PvP with a specific player
	 * 
	 * @param player
	 *            Player
	 */
	public void denyPvp(OfflinePlayer player) {
		pvp.put(player.getName().toLowerCase(), false);
		if (player.getPlayer() != null && this.player.getPlayer() != null) {
			TagAPI.refreshPlayer(this.player.getPlayer(), player.getPlayer());
			TabAPI.updateTablist(player.getPlayer());
		}
	}

	/**
	 * Get color format for a specific player
	 * 
	 * @param player
	 *            Player
	 * @return Chat Color
	 */
	public ChatColor getColorFormat(Player player) {
		return getStatus(player).getColor();
	}

	/**
	 * Get global color format
	 * 
	 * @return Chat Color
	 */
	public ChatColor getGlobalColorFormat() {
		return globalStatus.getColor();
	}

	/**
	 * Get Global pvp
	 * 
	 * @return Global PVP
	 */
	public boolean getGlobalPvp() {
		return pvpGlobal;
	}

	/**
	 * Set Global pvp
	 * 
	 * @param pvp
	 *            Global PVP
	 */
	public void setGlobalPvp(boolean pvp) {
		this.pvpGlobal = pvp;
		if (this.player.getPlayer() != null) {
			TagAPI.refreshPlayer(this.player.getPlayer());
			TabAPI.updateTablists();
		}
	}

	/**
	 * Set the pvp state
	 * 
	 * @param player
	 *            Player
	 * @param state
	 *            State
	 */
	public void setPvp(OfflinePlayer player, boolean state) {
		if (state == true) {
			allowPvp(player);
		} else {
			denyPvp(player);
		}
	}

	/**
	 * Clear the specific PVP setting
	 * 
	 * @param player
	 *            Player
	 */
	public void clearPvp(OfflinePlayer player) {
		if (pvp.containsKey(player.getName().toLowerCase())) {
			pvp.remove(player.getName().toLowerCase());
			if (player.getPlayer() != null && this.player.getPlayer() != null) {
				TagAPI.refreshPlayer(this.player.getPlayer(),
						player.getPlayer());
				TabAPI.updateTablist(player.getPlayer());
			}
		}
	}

	/**
	 * Clear all pvp settings
	 */
	public void clearPvp() {
		pvp = new TreeMap<String, Boolean>();
		if (this.player.getPlayer() != null) {
			TagAPI.refreshPlayer(this.player.getPlayer());
			TabAPI.updateTablists();
		}
	}

	/**
	 * Clear the status
	 * 
	 * @param player
	 *            Player
	 */
	public void clearStatus(OfflinePlayer player) {
		if (playerStatus.containsKey(player.getName().toLowerCase())) {
			playerStatus.remove(player.getName().toLowerCase());
			if (player.getPlayer() != null && this.player.getPlayer() != null) {
				TagAPI.refreshPlayer(this.player.getPlayer(),
						player.getPlayer());
				TabAPI.updateTablist(player.getPlayer());
			}
		}
	}

	/**
	 * Clear all statuses
	 */
	public void clearStatus() {
		playerStatus = new TreeMap<String, ACStatus>();
		if (this.player.getPlayer() != null) {
			TagAPI.refreshPlayer(this.player.getPlayer());
			TabAPI.updateTablists();
		}
	}

	/**
	 * Check if PVP with specific player
	 * 
	 * @param player
	 *            Player
	 * @return Boolean
	 */
	public boolean getPvp(OfflinePlayer player) {
		if (pvp.containsKey(player.getName().toLowerCase())) {
			return pvp.get(player.getName().toLowerCase()); // Get pvp
		} else {
			return pvpGlobal;
		}
	}
}
