package be.maximvdw.ac;

import java.util.Set;
import java.util.TreeMap;

import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.plugin.PluginManager;

import be.maximvdw.ac.chat.ChatFormatter;
import be.maximvdw.ac.commands.AutismChatCommand;
import be.maximvdw.ac.commands.GlobalChatCommand;
import be.maximvdw.ac.commands.GreenCommand;
import be.maximvdw.ac.commands.LoginResetCommand;
import be.maximvdw.ac.commands.MessageCommand;
import be.maximvdw.ac.commands.PvpCommand;
import be.maximvdw.ac.commands.RedCommand;
import be.maximvdw.ac.commands.ReplyCommand;
import be.maximvdw.ac.commands.SpecialCommand;
import be.maximvdw.ac.commands.WhiteCommand;
import be.maximvdw.ac.config.Configuration;
import be.maximvdw.ac.lib.MVdWPlugin;
import be.maximvdw.ac.listeners.EntityListener;
import be.maximvdw.ac.listeners.PlayerListener;
import be.maximvdw.ac.storage.YamlStorage;
import be.maximvdw.ac.tab.TabAPI;
import be.maximvdw.ac.ui.SendConsole;

/**
 * AutismChat
 * 
 * The main plugin instance. Controls the loading of the plugin and the
 * datastorage for the player information.
 * 
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class AutismChat extends MVdWPlugin {
	// Chat players
	TreeMap<String, ACPlayer> players = new TreeMap<String, ACPlayer>();
	YamlStorage storage = null; // Yaml Storage
	ChatFormatter chatFormatter = null; // Chat formatter
	public static String prefixGood = "&6[&9AutismChat&6] &8"; // Good prefix
	public static String prefixBad = "&6[&cAutismChat&6] &c"; // Bad prefix

	/**
	 * On plugin enable
	 */
	@Override
	public void onEnable() {
		new SendConsole(this); // Load logger

		// MOTD
		SendConsole.info("-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-");
		SendConsole.info("Plugin: AutismChat");
		SendConsole.info("Author: Maximvdw");
		SendConsole.info("Site: http://www.mvdw-software.be/");
		SendConsole.info("-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-");

		new TabAPI(this);
		chatFormatter = new ChatFormatter(this);
		loadConfig();
		registerListeners(Bukkit.getPluginManager());
		registerCommands();
	}

	/**
	 * Load configuration
	 */
	public void loadConfig() {
		SendConsole.info("Loading configuration ...");
		try {
			storage = new YamlStorage(this, "players"); // Load storage
			storage.loadConfig(); // Load storage
			new Configuration(this);
			Configuration.loadConfig();
			Configuration.readConfig();
			loadACPlayers(); // Load players
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * On plugin disable
	 */
	@Override
	public void onDisable() {
		saveACPlayers(); // Save AC Players
	}

	/**
	 * Get check formatter
	 * 
	 * @return Chat Formatter
	 */
	public ChatFormatter getChatFormatter() {
		return chatFormatter;
	}

	/**
	 * Get chat player
	 * 
	 * @param player
	 *            Player
	 * @return Chat Player
	 */
	public ACPlayer getACPlayer(Player player) {
		return getACPlayer(player.getName());
	}

	/**
	 * Check if ac player exists
	 * 
	 * @param player
	 *            Player
	 * @return boolean
	 */
	public boolean acPlayerExists(Player player) {
		if (players.containsKey(player.getName().toLowerCase()))
			return true;
		else
			return false;
	}

	/**
	 * Set AC Player
	 * 
	 * @param player
	 *            ACPlayer
	 */
	public void setACPlayer(ACPlayer player) {
		players.put(player.getPlayer().getName().toLowerCase(), player);
	}

	/**
	 * Get chat player
	 * 
	 * @param player
	 *            Player
	 * @return Chat Player
	 */
	public ACPlayer getACPlayer(String player) {
		if (players.containsKey(player.toLowerCase()))
			return players.get(player.toLowerCase());
		else {
			ACPlayer acPlayer = new ACPlayer(Bukkit.getPlayer(player));
			acPlayer.setDefaultStatus(Configuration.defaultStatus);
			players.put(player.toLowerCase(), acPlayer);
			acPlayer.setGlobalStatus(Configuration.defaultStatus);
			return acPlayer;
		}
	}

	/**
	 * Register Listeners
	 */
	@Override
	public void registerListeners(PluginManager pm) {
		pm.registerEvents(new PlayerListener(this), this);
		pm.registerEvents(new EntityListener(this), this);
	}

	/**
	 * Save AC Players
	 */
	public void saveACPlayers() {
		SendConsole.info("Saving player information...");
		// Get the configuration
		YamlConfiguration config = storage.getConfig();
		for (ACPlayer player : players.values()) {
			try {
				ACStatus globalStatus = player.getGlobalStatus();
				TreeMap<String, ACStatus> playerStatus = player
						.getPlayerSpecificStatuses();
				TreeMap<String, Boolean> pvp = player.getPlayerSpeificPvp();
				String prefix = "players."
						+ player.getPlayer().getName().toLowerCase() + ".";
				config.set(prefix + "globalStatus", globalStatus.name());
				config.set(prefix + "globalChat", player.getGlobalChat());
				config.set(prefix + "globalPvp", player.getGlobalPvp());
				config.set(prefix + "loginReset", player.getLoginReset());
				config.set(prefix + "defaultStatus", player.getDefaultStatus()
						.name());
				config.set(prefix + "status", null); // Reset status
														// players
				config.set(prefix + "pvp", null); // Reset pvp players
				for (String playerName : playerStatus.keySet()) {
					ACStatus status = playerStatus.get(playerName);
					config.set(prefix + "status." + playerName, status.name());
				}
				for (String playerName : pvp.keySet()) {
					boolean pvpState = pvp.get(playerName);
					config.set(prefix + "pvp." + playerName, pvpState);
				}
				config.save(storage.getConfigFile());
			} catch (Exception ex) {
				// Error
			}
		}
	}

	/**
	 * Load AC Players
	 */
	public void loadACPlayers() {
		SendConsole.info("Loading player information...");
		// Get the configuration
		YamlConfiguration config = storage.getConfig();
		try {
			Set<String> playerNames = config.getConfigurationSection("players")
					.getKeys(false);
			SendConsole.debug("Total players in config: " + playerNames.size());
			for (String playerName : playerNames) {
				try {
					SendConsole.debug("Loading information for " + playerName);
					String prefix = "players." + playerName;
					OfflinePlayer player = Bukkit.getOfflinePlayer(playerName);
					ACPlayer acPlayer = new ACPlayer(player); // Create player
					boolean reset = config.getBoolean(prefix + ".loginReset");
					if (reset)
						acPlayer.setResetOnLogin();
					else {
						ACStatus globalStatus = ACStatus.valueOf(config
								.getString(prefix + ".globalStatus"));
						SendConsole.debug(playerName + " - globalStatus: "
								+ globalStatus.name());
						acPlayer.setGlobalStatus(globalStatus);
						acPlayer.setGlobalChat(config.getBoolean(prefix
								+ ".globalChat"));
						SendConsole.debug(playerName + " - globalChat: "
								+ acPlayer.getGlobalChat());
						acPlayer.setGlobalPvp(config.getBoolean(prefix
								+ ".globalPvp"));
						acPlayer.setDefaultStatus(ACStatus.valueOf(config
								.getString(prefix + ".defaultStatus")));
						SendConsole.debug(playerName + " - globalPvp: "
								+ acPlayer.getGlobalPvp());
						ConfigurationSection sectionStatus = config
								.getConfigurationSection(prefix + ".status");
						if (sectionStatus != null) {
							Set<String> statusPlayers = sectionStatus
									.getKeys(false);
							for (String statusPlayer : statusPlayers) {
								ACStatus status = ACStatus.valueOf(config
										.getString(prefix + ".status."
												+ statusPlayer));
								SendConsole.debug(playerName + " - Status["
										+ statusPlayer + "]: " + status.name());
								OfflinePlayer target = Bukkit
										.getOfflinePlayer(statusPlayer);
								acPlayer.setStatus(status, target); // Set
																	// status
							}
						}

						ConfigurationSection sectionPvp = config
								.getConfigurationSection("players."
										+ playerName + ".pvp");
						if (sectionPvp != null) {
							Set<String> pvpPlayers = sectionPvp.getKeys(false);
							for (String pvpPlayer : pvpPlayers) {
								OfflinePlayer target = Bukkit
										.getOfflinePlayer(pvpPlayer);
								acPlayer.setPvp(
										target,
										config.getBoolean("players."
												+ playerName + ".pvp."
												+ pvpPlayer));
								SendConsole.debug(playerName + " - Pvp["
										+ pvpPlayer + "]: "
										+ acPlayer.getPvp(target));
							}
						}
					}
					this.players.put(playerName, acPlayer); // Save player
				} catch (Exception ex) {
					// Error
				}
			}
		} catch (Exception ex) {
			// Error
		}
	}

	/**
	 * Register commands
	 */
	@Override
	public void registerCommands() {
		try {
			getCommand("white").setExecutor(new WhiteCommand(this));
			getCommand("green").setExecutor(new GreenCommand(this));
			getCommand("red").setExecutor(new RedCommand(this));
			getCommand("pvp").setExecutor(new PvpCommand(this));
			getCommand("msg").setExecutor(new MessageCommand(this));
			getCommand("reply").setExecutor(new ReplyCommand(this));
			getCommand("globalchat").setExecutor(new GlobalChatCommand(this));
			getCommand("special").setExecutor(new SpecialCommand(this));
			getCommand("loginreset").setExecutor(new LoginResetCommand(this));
			getCommand("autismchat").setExecutor(new AutismChatCommand(this));
		} catch (Exception ex) {
			// Error
			ex.printStackTrace();
		}
	}
}
