package be.maximvdw.ac.chat;

import org.bukkit.ChatColor;
import org.bukkit.entity.Player;

import be.maximvdw.ac.ACPlayer;
import be.maximvdw.ac.AutismChat;
import be.maximvdw.ac.config.Configuration;
import be.maximvdw.ac.utils.ColorUtils;

/**
 * ChatFormatter
 * 
 * This class will arrange the formatting for the chat
 * 
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class ChatFormatter {
	private String format = "<%name%> %message%"; // Chat format
	private AutismChat plugin = null; // Plugin instance

	/**
	 * Initialize chat formatter
	 * 
	 * @param plugin
	 *            Plugin instance
	 */
	public ChatFormatter(AutismChat plugin) {
		this.plugin = plugin;
	}

	/**
	 * Get raw chat format string
	 * 
	 * @return Raw format
	 */
	public String getRawChatFormat() {
		return format;
	}

	/**
	 * Set raw chat format string
	 * 
	 * @param format
	 *            Raw format
	 */
	public void setRawChatFormat(String format) {
		this.format = format;
	}

	/**
	 * Get chat format
	 * 
	 * @param message
	 *            Message
	 * @param player
	 *            Player
	 * @param target
	 *            Target player
	 * @return Chat format
	 */
	public String getChatFormat(String message, Player player, Player target) {
		ACPlayer acPlayer = plugin.getACPlayer(player); // Get ac player
		ChatColor prefixColor = acPlayer.getColorFormat(target); // Get color
		String chatName = "";
		if (!Configuration.getConfig().getBoolean("format-chat.disable-color")) {
			chatName = prefixColor + "";
		}
		if (acPlayer.getPvp(target)
				&& !Configuration.getConfig().getBoolean(
						"format-chat.disable-pvp")) {
			chatName += "&l";
		}
		chatName += player.getName() + ChatColor.RESET; // Chatname
		String format = getRawChatFormat();
		format = format.replace("%name%", chatName);
		format = format.replace("%message%", message);
		format = ColorUtils.toColors(format); // Conver to colors
		return format; // Return chat format
	}
}
