package be.maximvdw.ac.chat;

import org.bukkit.entity.Player;
import org.bukkit.event.Event;
import org.bukkit.event.HandlerList;

/**
 * PlayerMessageEvent
 * 
 * Player private message event
 * 
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class PlayerMessageEvent extends Event {
	private Player sender = null; // Sender
	private Player receiver = null; // Receiver
	private String message = ""; // Message
	/**
	 * Event listener handler list.
	 */
	private static final HandlerList handlers = new HandlerList();

	/**
	 * Create a new player message event
	 * 
	 * @param sender
	 *            Sender Player
	 * @param receiver
	 *            Receiver Player
	 * @param message
	 *            Message
	 */
	public PlayerMessageEvent(Player sender, Player receiver, String message) {
		this.sender = sender;
		this.receiver = receiver;
		this.message = message;
	}

	/**
	 * Get sender
	 * 
	 * @return Player
	 */
	public Player getSender() {
		return sender;
	}

	/**
	 * Get receiver
	 * 
	 * @return Player
	 */
	public Player getReceiver() {
		return receiver;
	}

	/**
	 * Get message
	 * 
	 * @return String Message
	 */
	public String getMessage() {
		return message;
	}

	/**
	 * Get Handlers
	 */
	@Override
	public HandlerList getHandlers() {
		return handlers;
	}

	/**
	 * Get Handlers list
	 * 
	 * @return HandlerList
	 */
	public static HandlerList getHandlerList() {
		return handlers;
	}
}
