package be.maximvdw.ac.commands;

import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import be.maximvdw.ac.ACPlayer;
import be.maximvdw.ac.ACStatus;
import be.maximvdw.ac.AutismChat;
import be.maximvdw.ac.lang.Messages;
import be.maximvdw.ac.ui.SendConsole;
import be.maximvdw.ac.ui.SendGame;
import be.maximvdw.ac.utils.Permissions;
import be.maximvdw.ac.utils.Permissions.Permission;

/**
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class GlobalChatCommand implements CommandExecutor {
	AutismChat plugin = null; // Plugin instance

	/**
	 * Initialize the command
	 * 
	 * @param plugin
	 *            Plugin instance
	 */
	public GlobalChatCommand(AutismChat plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean onCommand(CommandSender sender, Command cmd, String label,
			String[] args) {
		Player player = null; // Player
		if (sender instanceof Player) {
			player = (Player) sender; // Cast to player
		}

		// Check if player
		if (player == null) {
			SendConsole.warning("This command can only be used ingame!");
			return false; // Return
		}

		// Check if has permisisons
		if (!Permissions.hasPermissions(Permission.GlobalChat, sender)) {
			SendGame.toPlayer(Messages.errNoPerm, player);
			return false;
		}

		ACPlayer acPlayer = plugin.getACPlayer(player); // Get Player

		if (args.length >= 1) {
			if (args[0].equalsIgnoreCase("on")) {
				acPlayer.setGlobalChat(true);
				SendGame.toPlayer(
						AutismChat.prefixGood + Messages.globalChatOn, player);
			} else if (args[0].equalsIgnoreCase("off")) {
				acPlayer.setGlobalChat(false);
				SendGame.toPlayer(AutismChat.prefixGood
						+ Messages.globalChatOff, player);
			} else {
				// Send a global message
				String message = args[0];
				for (int i = 1; i < args.length; i++) {
					message += " " + args[i];
				}
				player.chat(message);
				acPlayer.setGlobalStatus(ACStatus.WHITE);
			}
		} else {
			if (acPlayer.getGlobalChat()) {
				acPlayer.setGlobalChat(false);
				SendGame.toPlayer(AutismChat.prefixGood
						+ Messages.globalChatOff, player);
			} else {
				acPlayer.setGlobalChat(true);
				SendGame.toPlayer(
						AutismChat.prefixGood + Messages.globalChatOn, player);
			}
		}

		return false;
	}

}
