package be.maximvdw.ac.commands;

import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import be.maximvdw.ac.ACPlayer;
import be.maximvdw.ac.ACStatus;
import be.maximvdw.ac.AutismChat;
import be.maximvdw.ac.lang.Messages;
import be.maximvdw.ac.ui.SendConsole;
import be.maximvdw.ac.ui.SendGame;
import be.maximvdw.ac.utils.Permissions;
import be.maximvdw.ac.utils.Permissions.Permission;

/**
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class LoginResetCommand implements CommandExecutor {
	AutismChat plugin = null; // Plugin instance

	/**
	 * Initialize the command
	 * 
	 * @param plugin
	 *            Plugin instance
	 */
	public LoginResetCommand(AutismChat plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean onCommand(CommandSender sender, Command cmd, String label,
			String[] args) {
		Player player = null; // Player
		if (sender instanceof Player) {
			player = (Player) sender; // Cast to player
		}

		// Check if player
		if (player == null) {
			SendConsole.warning("This command can only be used ingame!");
			return false; // Return
		}

		// Check if has permisisons
		if (!Permissions.hasPermissions(Permission.LoginReset, sender)) {
			SendGame.toPlayer(Messages.errNoPerm, player);
			return false;
		}

		ACPlayer acPlayer = plugin.getACPlayer(player); // Get AC player
		boolean status = acPlayer.getLoginReset(); // Get reset
		if (args.length >= 1) {
			if (args[0].equalsIgnoreCase("on")) {
				acPlayer.setResetOnLogin();
				SendGame.toPlayer(AutismChat.prefixGood
						+ Messages.loginResetReset, player);
			} else if (args[0].equalsIgnoreCase("off")) {
				acPlayer.setKeepOnLogin();
				SendGame.toPlayer(AutismChat.prefixGood
						+ Messages.loginResetKeep, player);
			} else {
				try {
					acPlayer.setDefaultStatus(ACStatus.valueOf(args[0]
							.toUpperCase()));
					acPlayer.setResetOnLogin();
					String message = Messages.settingsReset;
					message = message.replace("{COLOR}", acPlayer
							.getDefaultStatus().getColor() + "");
					message = message.replace("{STATUS}", acPlayer
							.getDefaultStatus().name());
					SendGame.toPlayer(AutismChat.prefixGood + message, player);
				} catch (Exception ex) {
					// Error no such color
					SendGame.toPlayer(AutismChat.prefixBad
							+ Messages.noSuchColor.replace("{COLOR}", args[0]),
							player);
				}
			}
		} else {
			if (status == true) {
				acPlayer.setKeepOnLogin();
				SendGame.toPlayer(AutismChat.prefixGood
						+ Messages.loginResetKeep, player);
			} else {
				acPlayer.setResetOnLogin();
				SendGame.toPlayer(AutismChat.prefixGood
						+ Messages.loginResetReset, player);
			}
		}
		return false;
	}
}
