package be.maximvdw.ac.commands;

import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import be.maximvdw.ac.ACPlayer;
import be.maximvdw.ac.ACStatus;
import be.maximvdw.ac.AutismChat;
import be.maximvdw.ac.Channel;
import be.maximvdw.ac.chat.PlayerMessageEvent;
import be.maximvdw.ac.lang.Messages;
import be.maximvdw.ac.ui.SendConsole;
import be.maximvdw.ac.ui.SendGame;
import be.maximvdw.ac.utils.Permissions;
import be.maximvdw.ac.utils.Permissions.Permission;

/**
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class MessageCommand implements CommandExecutor {
	AutismChat plugin = null; // Plugin instance

	/**
	 * Initialize the command
	 * 
	 * @param plugin
	 *            Plugin instance
	 */
	public MessageCommand(AutismChat plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean onCommand(CommandSender sender, Command cmd, String label,
			String[] args) {
		Player player = null; // Player
		if (sender instanceof Player) {
			player = (Player) sender; // Cast to player
		}

		// Check if player
		if (player == null) {
			SendConsole.warning("This command can only be used ingame!");
			return false; // Return
		}

		// Check if has permisisons
		if (!Permissions.hasPermissions(Permission.Message, sender)) {
			SendGame.toPlayer(Messages.errNoPerm, player);
			return false;
		}

		ACPlayer acPlayer = plugin.getACPlayer(player); // Get player

		// Check if arguments
		if (args.length >= 1) {
			Player target = Bukkit.getPlayer(args[0]); // Get target
			if (target == null) {
				// Show error
				String message = Messages.notValidPlayer;
				message = message.replace("{TARGET}", args[0]);
				SendGame.toPlayer(AutismChat.prefixBad + message, player);
				return false;
			}

			if (args.length >= 2) {
				String message = args[1];
				for (int i = 2; i < args.length; i++) {
					message += " " + args[i];
				}

				// Set the status
				if (acPlayer.getStatus(target) == ACStatus.RED) {
					acPlayer.setStatus(ACStatus.WHITE, target);
					String msg = Messages.privateMessageWhite;
					msg = msg.replace("{TARGET}", target.getName());
					SendGame.toPlayer(AutismChat.prefixGood + msg, player);
				}
				PlayerMessageEvent event = new PlayerMessageEvent(player,
						target, message);
				plugin.getServer().getPluginManager().callEvent(event);
				return true;
			} else {
				if (acPlayer.getChatChannel() == Channel.GLOBAL) {
					acPlayer.setChatChannel(Channel.PRIVATE);
					acPlayer.setLastPMPlayer(target); // Last PM Player
					String message = Messages.privateMessageLocked;
					message = message.replace("{TARGET}", target.getName());
					SendGame.toPlayer(AutismChat.prefixGood + message, player);
				} else {
					acPlayer.setChatChannel(Channel.GLOBAL);
					String message = Messages.privateMessageUnlocked;
					message = message.replace("{TARGET}", target.getName());
					SendGame.toPlayer(AutismChat.prefixGood + message, player);
				}
			}
		} else {
			if (acPlayer.getChatChannel() == Channel.PRIVATE) {
				acPlayer.setChatChannel(Channel.GLOBAL);
				if (acPlayer.getLastPMPlayer() == null) {
					SendGame.toPlayer(AutismChat.prefixBad
							+ Messages.invalidArgs
							+ "/msg <player> &oYour message.", player);
					return false;
				}
				String message = Messages.privateMessageUnlocked;
				message = message.replace("{TARGET}", acPlayer
						.getLastPMPlayer().getName());
				SendGame.toPlayer(AutismChat.prefixGood + message, player);
			} else
				SendGame.toPlayer(AutismChat.prefixBad + Messages.invalidArgs
						+ "/msg <player> &oYour message.", player);
		}
		return false;
	}

}
