package be.maximvdw.ac.commands;

import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import be.maximvdw.ac.ACPlayer;
import be.maximvdw.ac.AutismChat;
import be.maximvdw.ac.lang.Messages;
import be.maximvdw.ac.ui.SendConsole;
import be.maximvdw.ac.ui.SendGame;
import be.maximvdw.ac.utils.Permissions;
import be.maximvdw.ac.utils.Permissions.Permission;

/**
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class PvpCommand implements CommandExecutor {
	AutismChat plugin = null; // Plugin instance

	/**
	 * Initialize the command
	 * 
	 * @param plugin
	 *            Plugin instance
	 */
	public PvpCommand(AutismChat plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean onCommand(CommandSender sender, Command cmd, String label,
			String[] args) {
		Player player = null; // Player
		if (sender instanceof Player) {
			player = (Player) sender; // Cast to player
		}

		// Check if player
		if (player == null) {
			SendConsole.warning("This command can only be used ingame!");
			return false; // Return
		}

		// Check if has permisisons
		if (!Permissions.hasPermissions(Permission.StatusPvp, sender)) {
			SendGame.toPlayer(Messages.errNoPerm, player);
			return false;
		}

		ACPlayer acPlayer = plugin.getACPlayer(player); // Ac Player

		// Check if arguments
		if (args.length >= 1) {
			boolean status = false; // Pvp status
			if (args[0].equalsIgnoreCase("on")) {
				acPlayer.setGlobalPvp(true);
				SendGame.toPlayer(AutismChat.prefixGood + Messages.globalPvpOn,
						player);
			} else if (args[0].equalsIgnoreCase("off")) {
				acPlayer.setGlobalPvp(true);
				SendGame.toPlayer(
						AutismChat.prefixGood + Messages.globalPvpOff, player);
			} else {
				String targetName = args[0]; // Get Player name from args
				OfflinePlayer target = Bukkit.getOfflinePlayer(targetName); // Target
				if (target == null) {
					// Show error
					String message = Messages.notValidPlayer;
					message = message.replace("{TARGET}", targetName);
					SendGame.toPlayer(AutismChat.prefixBad + message, player);
					return false;
				}

				if (args.length >= 2) {
					if (args[1].equalsIgnoreCase("on")) {
						status = true;
						String message = Messages.allowPvpPlayer;
						message = message.replace("{TARGET}", target.getName());
						SendGame.toPlayer(AutismChat.prefixGood + message,
								player);
					} else if (args[1].equalsIgnoreCase("off")) {
						status = false;
						String message = Messages.denyPvpPlayer;
						message = message.replace("{TARGET}", target.getName());
						SendGame.toPlayer(AutismChat.prefixGood + message,
								player);
					} else {
						// Error
						SendGame.toPlayer(
								AutismChat.prefixBad
										+ "Invalid command usage: /pvp <player> on|off",
								player);
						return false;
					}

					// Set pvp state
					acPlayer.setPvp(target, status);
				} else {
					// Get PVP status
					if (acPlayer.getPvp(target)) {
						status = false;
					} else {
						status = true;
					}

					if (status == true) {
						String message = Messages.allowPvpPlayer;
						message = message.replace("{TARGET}", target.getName());
						SendGame.toPlayer(AutismChat.prefixGood + message,
								player);
					} else {
						String message = Messages.denyPvpPlayer;
						message = message.replace("{TARGET}", target.getName());
						SendGame.toPlayer(AutismChat.prefixGood + message,
								player);
					}
					// Set pvp state
					acPlayer.setPvp(target, status);
				}
			}
		} else {
			// Set global pvp status
			if (acPlayer.getGlobalPvp()) {
				acPlayer.setGlobalPvp(false);
				SendGame.toPlayer(
						AutismChat.prefixGood + Messages.globalPvpOff, player);
			} else {
				acPlayer.setGlobalPvp(true);
				SendGame.toPlayer(AutismChat.prefixGood + Messages.globalPvpOn,
						player);
			}
		}
		return false;
	}

}
