package be.maximvdw.ac.commands;

import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import be.maximvdw.ac.ACPlayer;
import be.maximvdw.ac.ACStatus;
import be.maximvdw.ac.AutismChat;
import be.maximvdw.ac.lang.Messages;
import be.maximvdw.ac.ui.SendConsole;
import be.maximvdw.ac.ui.SendGame;
import be.maximvdw.ac.utils.Permissions;
import be.maximvdw.ac.utils.Permissions.Permission;

/**
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class RedCommand implements CommandExecutor {
	AutismChat plugin = null; // Plugin instance

	/**
	 * Initialize the command
	 * 
	 * @param plugin
	 *            Plugin instance
	 */
	public RedCommand(AutismChat plugin) {
		this.plugin = plugin;
	}

	@Override
	public boolean onCommand(CommandSender sender, Command cmd, String label,
			String[] args) {
		Player player = null; // Player
		if (sender instanceof Player) {
			player = (Player) sender; // Cast to player
		}

		// Check if player
		if (player == null) {
			SendConsole.warning("This command can only be used ingame!");
			return false; // Return
		}

		// Check if has permisisons
		if (!Permissions.hasPermissions(Permission.StatusRed, sender)) {
			SendGame.toPlayer(Messages.errNoPerm, player);
			return false;
		}

		ACStatus status = ACStatus.RED; // Status

		// Check if arguments
		if (args.length >= 1) {
			OfflinePlayer target = Bukkit.getOfflinePlayer(args[0]); // Get
			// target
			if (target == null) {
				// Show error
				String message = Messages.notValidPlayer;
				message = message.replace("{TARGET}", args[0]);
				SendGame.toPlayer(AutismChat.prefixBad + message, player);
				return false;
			}
			ACPlayer acPlayer = plugin.getACPlayer(player); // Get AC Player
			acPlayer.setStatus(status, target);

			String message = Messages.statusSet;
			message = message.replace("{COLOR}", status.getColor() + "");
			message = message.replace("{STATUS}", status.name());
			message = message.replace("{TARGET}", target.getName());
			SendGame.toPlayer(AutismChat.prefixGood + message, player);
		} else {
			ACPlayer acPlayer = plugin.getACPlayer(player); // Get AC Player
			acPlayer.setGlobalStatus(status);

			String message = Messages.statusSetGlobal;
			message = message.replace("{COLOR}", status.getColor() + "");
			message = message.replace("{STATUS}", status.name());
			SendGame.toPlayer(AutismChat.prefixGood + message, player);
		}
		return false;
	}

}
