package be.maximvdw.ac.commands;

import java.util.TreeMap;

import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import be.maximvdw.ac.ACPlayer;
import be.maximvdw.ac.ACStatus;
import be.maximvdw.ac.AutismChat;
import be.maximvdw.ac.lang.Messages;
import be.maximvdw.ac.ui.SendConsole;
import be.maximvdw.ac.ui.SendGame;

/**
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class SpecialCommand implements CommandExecutor {
	AutismChat plugin = null; // Plugin instance

	/**
	 * Initialize the command
	 * 
	 * @param plugin
	 *            Plugin instance
	 */
	public SpecialCommand(AutismChat plugin) {
		this.plugin = plugin;
	}

	/**
	 * List specials
	 * 
	 * @param player
	 *            AC Player
	 */
	public void listSpecial(ACPlayer player) {
		TreeMap<String, Boolean> pvp = player.getPlayerSpeificPvp();
		TreeMap<String, ACStatus> status = player.getPlayerSpecificStatuses();
		SendGame.toPlayer(Messages.playerSpecifics, player.getPlayer());
		for (String playerName : pvp.keySet()) {
			boolean pvpState = pvp.get(playerName);
			OfflinePlayer p = Bukkit.getOfflinePlayer(playerName);
			String cleanName = p.getName();
			SendGame.toPlayer(
					"&7"
							+ cleanName
							+ "&8 : "
							+ (status.containsKey(playerName) ? status
									.get(playerName) + " " : "")
							+ ((pvpState == true) ? "PVP enabled"
									: "PVP disabled"), player.getPlayer());
			if (status.containsKey(playerName))
				status.remove(playerName);
		}
		for (String playerName : status.keySet()) {
			ACStatus state = status.get(playerName);
			OfflinePlayer p = Bukkit.getOfflinePlayer(playerName);
			String cleanName = p.getName();
			SendGame.toPlayer("&7" + cleanName + "&8 : " + state.name(),
					player.getPlayer());
		}
	}

	@Override
	public boolean onCommand(CommandSender sender, Command cmd, String label,
			String[] args) {
		Player player = null; // Player
		if (sender instanceof Player) {
			player = (Player) sender; // Cast to player
		}

		// Check if player
		if (player == null) {
			SendConsole.warning("This command can only be used ingame!");
			return false; // Return
		}

		ACPlayer acPlayer = plugin.getACPlayer(player); // Get AC player

		if (args.length >= 1) {
			if (args[0].equalsIgnoreCase("clear")) {
				if (args.length == 2) {
					if (args[1].equals("all")) {
						acPlayer.clearStatus();
						acPlayer.clearPvp();
						SendGame.toPlayer(AutismChat.prefixGood
								+ Messages.clearedAllPlayers, player);
					} else {
						OfflinePlayer target = Bukkit.getOfflinePlayer(args[1]);
						if (target == null) {
							String message = Messages.notValidPlayer;
							message = message.replace("{TARGET}", args[1]);
							SendGame.toPlayer(AutismChat.prefixBad + message,
									player);
							return false;
						}

						acPlayer.clearPvp(target); // Clear PVP
						acPlayer.clearStatus(target); // Clear Status
						SendGame.toPlayer(
								AutismChat.prefixGood
										+ Messages.clearedPlayer.replace(
												"{TARGET}", target.getName()),
								player);
					}
				} else {
					SendGame.toPlayer(AutismChat.prefixBad
							+ Messages.invalidArgs
							+ "/special clear [<player>|all]", player);
					return false;
				}
			} else if (args[0].equalsIgnoreCase("list")) {
				listSpecial(acPlayer);
			} else {
				SendGame.toPlayer(AutismChat.prefixBad + Messages.invalidArgs
						+ "/special clear [<player>|all]", player);
				return false;
			}
		} else {
			SendGame.toPlayer(AutismChat.prefixBad + Messages.invalidArgs
					+ "/special clear|list", player);
		}
		return false;
	}
}
