package be.maximvdw.ac.config;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.bukkit.ChatColor;
import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.configuration.file.YamlConfiguration;

import be.maximvdw.ac.ACStatus;
import be.maximvdw.ac.AutismChat;
import be.maximvdw.ac.lang.Messages;

/**
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class Configuration {
	public static boolean debug = true; // Debug mode
	public static List<String> motd = new ArrayList<String>();
	public static boolean motdFirstJoin = true;
	public static ACStatus defaultStatus = ACStatus.WHITE;

	static AutismChat plugin = null; // Plugin instance

	/**
	 * Initialize the configuration
	 * 
	 * @param plugin
	 *            Plugin
	 */
	public Configuration(AutismChat plugin) {
		Configuration.plugin = plugin;
	}

	// Yaml Configuration
	private static YamlConfiguration pluginConfig;
	private static File configFile;
	private static boolean loaded = false;

	/**
	 * Gets the configuration file.
	 * 
	 * @return the myConfig
	 */
	public static YamlConfiguration getConfig() {
		if (!loaded) {
			loadConfig();
		}
		return pluginConfig;
	}

	/**
	 * Gets the configuration file.
	 * 
	 * @return Configuration file
	 */
	public static File getConfigFile() {
		return configFile;
	}

	/**
	 * Loads the configuration file from the .jar.
	 */
	public static void loadConfig() {
		configFile = new File(plugin.getDataFolder(), "config.yml");
		if (configFile.exists()) {
			pluginConfig = new YamlConfiguration();
			try {
				pluginConfig.load(configFile);
			} catch (FileNotFoundException ex) {
				// TODO: Log exception
			} catch (IOException ex) {
				// TODO: Log exception
			} catch (InvalidConfigurationException ex) {
				// TODO: Log exception
			}
			loaded = true;
		} else {
			try {
				plugin.getDataFolder().mkdir();
				InputStream jarURL = plugin.getClass().getResourceAsStream(
						"/config.yml");
				copyFile(jarURL, configFile);
				pluginConfig = new YamlConfiguration();
				pluginConfig.load(configFile);
				loaded = true;
				// TODO: Log that config has been loaded
			} catch (Exception e) {
				// TODO: Log exception
			}
		}
	}

	/**
	 * Read configuration
	 */
	public static void readConfig() {
		// Get colors
		try {
			ACStatus.WHITE.setColor(ChatColor.valueOf(getConfig().getString(
					"white").toUpperCase()));
			ACStatus.GREEN.setColor(ChatColor.valueOf(getConfig().getString(
					"green").toUpperCase()));
			ACStatus.RED.setColor(ChatColor.valueOf(getConfig()
					.getString("red").toUpperCase()));
			defaultStatus = ACStatus.valueOf(getConfig().getString("default")
					.toUpperCase());
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		AutismChat.prefixBad = getConfig().getString("messages.prefixBad");
		AutismChat.prefixGood = getConfig().getString("messages.prefixGood");
		Messages.errNoPerm = getConfig().getString("messages." + "errNoPerm");
		Messages.statusSet = getConfig().getString("messages." + "statusSet");
		Messages.statusSetGlobal = getConfig().getString(
				"messages." + "statusSetGlobal");
		Messages.notValidPlayer = getConfig().getString(
				"messages." + "notValidPlayer");
		Messages.settingsReset = getConfig().getString(
				"messages." + "settingsReset");
		Messages.globalChatOff = getConfig().getString(
				"messages." + "globalChatOff");
		Messages.globalChatOn = getConfig().getString(
				"messages." + "globalChatOn");
		Messages.globalPvpOn = getConfig().getString(
				"messages." + "globalPvpOn");
		Messages.globalPvpOff = getConfig().getString(
				"messages." + "globalPvpOff");
		Messages.allowPvpPlayer = getConfig().getString(
				"messages." + "allowPvpPlayer");
		Messages.denyPvpPlayer = getConfig().getString(
				"messages." + "denyPvpPlayer");
		Messages.loginResetKeep = getConfig().getString(
				"messages." + "loginResetKeep");
		Messages.loginResetReset = getConfig().getString(
				"messages." + "loginResetReset");
		Messages.noSuchColor = getConfig().getString(
				"messages." + "noSuchColor");
		Messages.noAccepting = getConfig().getString(
				"messages." + "noAccepting");
		Messages.invalidArgs = getConfig().getString(
				"messages." + "invalidArgs");
		Messages.privateMessageLocked = getConfig().getString(
				"messages." + "privateMessageLocked");
		Messages.privateMessageUnlocked = getConfig().getString(
				"messages." + "privateMessageUnlocked");
		Messages.privateMessageWhite = getConfig().getString(
				"messages." + "privateMessageWhite");
		Messages.replyNoMessages = getConfig().getString(
				"messages." + "replyNoMessages");
		Messages.clearedAllPlayers = getConfig().getString(
				"messages." + "clearedAllPlayers");
		Messages.clearedPlayer = getConfig().getString(
				"messages." + "clearedPlayer");
		Messages.playerSpecifics = getConfig().getString(
				"messages." + "playerSpecifics");

		debug = getConfig().getBoolean("debug");
		motd = getConfig().getStringList("motd"); // Get mod
		motdFirstJoin = getConfig().getBoolean("firstJoin");
		plugin.getChatFormatter().setRawChatFormat(
				getConfig().getString("chatformat"));
	}

	/**
	 * Copies a file to a new location.
	 * 
	 * @param in
	 *            InputStream
	 * @param out
	 *            File
	 * 
	 * @throws Exception
	 */
	static private void copyFile(InputStream in, File out) throws Exception {
		InputStream fis = in;
		FileOutputStream fos = new FileOutputStream(out);
		try {
			byte[] buf = new byte[1024];
			int i = 0;
			while ((i = fis.read(buf)) != -1) {
				fos.write(buf, 0, i);
			}
		} catch (Exception e) {
			throw e;
		} finally {
			if (fis != null) {
				fis.close();
			}
			if (fos != null) {
				fos.close();
			}
		}
	}
}
