package be.maximvdw.ac.listeners;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.player.AsyncPlayerChatEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.kitteh.tag.AsyncPlayerReceiveNameTagEvent;

import be.maximvdw.ac.ACPlayer;
import be.maximvdw.ac.ACStatus;
import be.maximvdw.ac.AutismChat;
import be.maximvdw.ac.Channel;
import be.maximvdw.ac.chat.ChatFormatter;
import be.maximvdw.ac.chat.PlayerMessageEvent;
import be.maximvdw.ac.config.Configuration;
import be.maximvdw.ac.lang.Messages;
import be.maximvdw.ac.tab.TabAPI;
import be.maximvdw.ac.ui.SendGame;
import be.maximvdw.ac.utils.ColorUtils;

/**
 * PlayerListener
 * 
 * This listener will listen to Tag, Message and chat events.
 * 
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class PlayerListener implements Listener {
	AutismChat plugin = null; // Plugin instance

	/**
	 * Initialize listener
	 * 
	 * @param plugin
	 *            Plugin instance
	 */
	public PlayerListener(AutismChat plugin) {
		this.plugin = plugin;
	}

	/**
	 * On Player join event
	 * 
	 * @param event
	 *            PlayerJoinEvent
	 */
	@EventHandler(priority = EventPriority.NORMAL)
	public void onPlayerJoin(PlayerJoinEvent event) {
		Player player = event.getPlayer(); // Get player

		// Motd
		if (!event.getPlayer().hasPlayedBefore()
				|| !Configuration.motdFirstJoin) {
			for (String line : Configuration.motd) {
				line = line.replace("{PLAYER}", player.getName());
				SendGame.toPlayer(line, player);
			}
		}
		ACPlayer acPlayer = plugin.getACPlayer(player);
		if (acPlayer.getLoginReset()) {
			acPlayer.setGlobalStatus(acPlayer.getDefaultStatus());
		}
		// Update the tablist for all players
		Bukkit.getScheduler().runTaskLater(plugin, new Runnable() {

			@Override
			public void run() {
				TabAPI.updateTablists();
			}

		}, 3L);
	}

	/**
	 * On Player quit event
	 * 
	 * @param event
	 *            PlayerQuitEvent
	 */
	@EventHandler(priority = EventPriority.NORMAL)
	public void onPlayerQuit(PlayerQuitEvent event) {
		// Update the tablist for all players
		Bukkit.getScheduler().runTaskLater(plugin, new Runnable() {

			@Override
			public void run() {
				TabAPI.updateTablists();
			}

		}, 3L);
	}

	/**
	 * On Player chat event
	 * 
	 * @param event
	 *            AsyncPlayerChatEvent
	 */
	@EventHandler(priority = EventPriority.NORMAL)
	public void onPlayerChat(AsyncPlayerChatEvent event) {
		if (event.isCancelled()) {
			return;
		}

		Player player = event.getPlayer(); // Get player
		ChatFormatter formatter = plugin.getChatFormatter();
		final ACPlayer acPlayer = plugin.getACPlayer(player);
		// Check channel
		if (acPlayer.getChatChannel() == Channel.PRIVATE) {
			Player target = acPlayer.getLastPMPlayer(); // Get pm
			PlayerMessageEvent e = new PlayerMessageEvent(player, target,
					event.getMessage());
			plugin.getServer().getPluginManager().callEvent(e);
			event.setCancelled(true); // Cancel
			return;
		}

		if (!acPlayer.getGlobalChat()) {
			acPlayer.setGlobalChatRaw(true); // Set raw
			Bukkit.getScheduler().runTask(plugin, new Runnable() {

				@Override
				public void run() {
					acPlayer.setGlobalChat(true); // Allow global chat
				}

			});
		}

		event.setCancelled(true); // Cancel event
		for (Player p : Bukkit.getOnlinePlayers()) {
			ACPlayer acTarget = plugin.getACPlayer(p);
			if (acTarget.getGlobalChat()) {
				String message = formatter.getChatFormat(event.getMessage(),
						player, p);
				SendGame.toPlayer(message, p); // Send message
			}
		}
	}

	/**
	 * On name tag
	 * 
	 * @param event
	 *            AsyncPlayerReceiveNameTagEvent
	 */
	@EventHandler
	public void onNameTag(AsyncPlayerReceiveNameTagEvent event) {
		Player player = event.getNamedPlayer(); // Player with the tag
		Player target = event.getPlayer(); // Get player who wants the tag
		ACPlayer acPlayer = plugin.getACPlayer(player); // Get AC Player
		ChatColor prefixColor = acPlayer.getColorFormat(target); // Get color
		String nameTag = "";
		if ((prefixColor != ChatColor.WHITE)
				&& !Configuration.getConfig().getBoolean(
						"format-tag.disable-color"))
			nameTag += prefixColor; // Create name Tag
		if (!acPlayer.getGlobalChat()
				&& !Configuration.getConfig().getBoolean(
						"format-tag.disable-global")) {
			nameTag += "&m";
		}
		if (acPlayer.getPvp(target)
				&& !Configuration.getConfig().getBoolean(
						"format-tag.disable-pvp")) {
			nameTag += "&l";
		}
		nameTag += player.getName();
		event.setTag(ColorUtils.toColors(nameTag)); // Set tag
	}

	/**
	 * Player private message event
	 * 
	 * @param event
	 *            PlayerMessageEvent
	 */
	@EventHandler
	public void onPrivateMessage(PlayerMessageEvent event) {
		Player sender = event.getSender(); // Get sender
		Player receiver = event.getReceiver(); // Get receiver
		String message = event.getMessage();
		String messageSender = "&7(To " + receiver.getName() + "):&f "
				+ message;
		String messageReceiver = "&7(From " + sender.getName() + "):&f "
				+ message;
		ACPlayer acReceiver = plugin.getACPlayer(receiver); // Get ac player
		acReceiver.setLastMessage(sender); // Set last message sender
		if (acReceiver.getStatus(sender) != ACStatus.RED) // Get status
		{
			SendGame.toPlayer(messageSender, sender);
			SendGame.toPlayer(messageReceiver, receiver);
		} else {
			SendGame.toPlayer(
					AutismChat.prefixBad
							+ Messages.noAccepting.replace("{RECEIVER}",
									receiver.getName()), sender);
		}
	}
}
