package be.maximvdw.ac.storage;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.bukkit.configuration.InvalidConfigurationException;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.Plugin;

/**
 * YamlStorage
 * 
 * YAML File storage
 * 
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class YamlStorage {
	Plugin plugin = null; // Plugin instance
	String configName = ""; // Configuration name

	/**
	 * Initialize the Yaml Configuration
	 * 
	 * @param plugin
	 *            Plugin
	 */
	public YamlStorage(Plugin plugin, String configName) {
		this.plugin = plugin;
		this.configName = configName;
	}

	// Yaml Configuration
	private YamlConfiguration pluginConfig;
	private File configFile;
	private boolean loaded = false;
	
	/**
	 * Gets the configuration file.
	 * 
	 * @return the myConfig
	 */
	public YamlConfiguration getConfig() {
		if (!loaded) {
			loadConfig();
		}
		return pluginConfig;
	}

	/**
	 * Gets the configuration file.
	 * 
	 * @return Configuration file
	 */
	public File getConfigFile() {
		return configFile;
	}

	/**
	 * Loads the configuration file
	 */
	public void loadConfig() {
		configFile = new File(plugin.getDataFolder(), configName + ".yml");
		if (configFile.exists()) {
			pluginConfig = new YamlConfiguration();
			try {
				pluginConfig.load(configFile);
			} catch (FileNotFoundException ex) {
				// TODO: Log exception
			} catch (IOException ex) {
				// TODO: Log exception
			} catch (InvalidConfigurationException ex) {
				// TODO: Log exception
			}
			loaded = true;
		} else {
			try {
				plugin.getDataFolder().mkdir();
				configFile.createNewFile();

				pluginConfig = new YamlConfiguration();
				pluginConfig.load(configFile);
				loaded = true;
				// TODO: Log that config has been loaded
			} catch (Exception e) {
				// TODO: Log exception
			}
		}
	}
}
