package be.maximvdw.ac.tab;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.entity.Player;
import org.bukkit.plugin.Plugin;

import be.maximvdw.ac.ACPlayer;
import be.maximvdw.ac.AutismChat;
import be.maximvdw.ac.config.Configuration;
import be.maximvdw.ac.utils.ColorUtils;

import com.comphenix.protocol.PacketType;
import com.comphenix.protocol.ProtocolLibrary;
import com.comphenix.protocol.ProtocolManager;
import com.comphenix.protocol.events.ListenerPriority;
import com.comphenix.protocol.events.PacketAdapter;
import com.comphenix.protocol.events.PacketContainer;
import com.comphenix.protocol.events.PacketEvent;

/**
 * TabAPI
 * 
 * A rewrite of the Tab API To function and work for specific players.
 * 
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class TabAPI {

	static AutismChat plugin = null; // Plugin instance
	private static HashMap<String, TabObject> playerTab = new HashMap<String, TabObject>();
	private static HashMap<String, TabHolder> playerTabLast = new HashMap<String, TabHolder>();

	private static HashMap<Player, ArrayList<PacketContainer>> cachedPackets = new HashMap<Player, ArrayList<PacketContainer>>();
	private static HashMap<Player, Integer> updateSchedules = new HashMap<Player, Integer>();

	private static int max = 0;

	private static String[] colors = { "1", "2", "3", "4", "5", "6", "7", "8",
			"9", "0", "a", "b", "c", "d", "f", "g", "h", "i", "j", "k", "l",
			"m", "n", "o", "p", "r", "s", "t", "u", "v", "w", "x", "y", "z" };

	private static int e = 0;
	private static int r = 0;

	private static long flickerPrevention = 5L;

	private static ProtocolManager protocolManager;

	private static boolean shuttingdown = false;

	public TabAPI(AutismChat plugin) {
		TabAPI.plugin = plugin;

		max = Bukkit.getServer().getMaxPlayers();
		protocolManager = ProtocolLibrary.getProtocolManager();

		for (Player p : Bukkit.getOnlinePlayers()) {
			resetTabList(p);
		}
		protocolManager.addPacketListener(new PacketAdapter(TabAPI.plugin,
				ListenerPriority.NORMAL, PacketType.Play.Server.PLAYER_INFO) {
			@Override
			public void onPacketSending(PacketEvent event) {
				if (event.getPacket().getType() == PacketType.Play.Server.PLAYER_INFO) {

					PacketContainer p = event.getPacket();
					String s = p.getStrings().read(0);
					if (s.startsWith("$")) {
						p.getStrings().write(0, s.substring(1));
						event.setPacket(p);
					} else {
						event.setCancelled(true);
					}
				}
			}
		});

	}

	/**
	 * Get max players
	 * 
	 * @return int Max players
	 */
	public static int getMaxPlayers() {
		return max;
	}

	public static void updateTablists() {
		for (Player player : Bukkit.getOnlinePlayers()) {
			TabAPI.updateTablist(player);
		}
	}

	public static void updateTablist(Player player) {
		TabObject tabo = new TabObject();
		playerTab.put(player.getName(), tabo);
		Player[] players = Bukkit.getOnlinePlayers();
		for (int i = 0; i < players.length; i++) {
			Player p = players[i]; // Get player
			ACPlayer acPlayer = TabAPI.plugin.getACPlayer(p); // Get AC Player
			ChatColor prefixColor = acPlayer.getColorFormat(player); // Get
																		// color
			String tabName = "";
			if ((prefixColor != ChatColor.WHITE)
					&& !Configuration.getConfig().getBoolean(
							"format-tab.disable-color"))
				tabName += prefixColor; // Create name Tag
			if (!acPlayer.getGlobalChat()
					&& !Configuration.getConfig().getBoolean(
							"format-tab.disable-global")) {
				tabName += "&m";
			}
			if (acPlayer.getPvp(player)
					&& !Configuration.getConfig().getBoolean(
							"format-tab.disable-pvp")) {
				tabName += "&l";
			}
			tabName += p.getName();
			TabAPI.setTabString(plugin, player, i, ColorUtils.toColors(tabName));
		}
		// Send updated tab list
		TabAPI.updatePlayer(player);
	}

	private static void addPacket(Player p, String msg, boolean b, int ping) {
		PacketContainer message = protocolManager
				.createPacket(PacketType.Play.Server.PLAYER_INFO);
		message.getStrings().write(0, ((!shuttingdown) ? "$" : "") + msg);
		message.getBooleans().write(0, b);
		message.getIntegers().write(0, ping);
		ArrayList<PacketContainer> packetList = cachedPackets.get(p);
		if (packetList == null) {
			packetList = new ArrayList<PacketContainer>();
			cachedPackets.put(p, packetList);
		}
		packetList.add(message);
	}

	private static void flushPackets(final Player p, final TabHolder tabCopy) {
		final PacketContainer[] packets = (PacketContainer[]) cachedPackets
				.get(p).toArray(new PacketContainer[0]);

		// cancel old task (prevents flickering)
		Integer taskID = updateSchedules.get(p);
		if (taskID != null) {
			Bukkit.getScheduler().cancelTask(taskID);
		}

		taskID = Bukkit.getScheduler().scheduleSyncDelayedTask(TabAPI.plugin,
				new Runnable() {
					@Override
					public void run() {
						if (p.isOnline()) {

							for (PacketContainer packet : packets) {
								try {
									protocolManager.sendServerPacket(p, packet);
								} catch (InvocationTargetException e) {
									e.printStackTrace();
								}
							}
						}
						if (tabCopy != null)
							playerTabLast.put(p.getName(), tabCopy);
						updateSchedules.remove(p);
					}
				}, flickerPrevention);

		// let's keep a reference to be able to cancel this (see above)
		updateSchedules.put(p, taskID);

		cachedPackets.remove(p);
	}

	private static TabObject getTab(Player p) {
		TabObject tabo = playerTab.get(p.getName());
		if (tabo == null) {
			tabo = new TabObject();
			playerTab.put(p.getName(), tabo);
		}
		return tabo;
	}

	/**
	 * Returns the tab list to the vanilla tab list for a player. If another
	 * plugin holds higher priority, this does notning
	 * 
	 * @param p
	 */
	public static void disableTabForPlayer(Player p) {
		playerTab.put(p.getName(), null);
		resetTabList(p);
	}

	/**
	 * Resets tab to normal tab.
	 * 
	 * @param p
	 */
	public static void resetTabList(Player p) {
		int pos = 0;
		for (Player pl : Bukkit.getOnlinePlayers()) {
			setTabString(plugin, p, pos, pl.getPlayerListName());
			pos++;
		}
	}

	public static void setTabString(Plugin plugin, Player p, int pos, String msg) {
		setTabString(plugin, p, pos, msg, 0);
	}

	/**
	 * Set the tab for a player.
	 * 
	 * If the plugin the tab is being set from does not have a priority, It will
	 * automatically be give a base priority of 0
	 * 
	 * @param plugin
	 * @param p
	 * @param x
	 * @param y
	 * @param msg
	 * @param ping
	 */
	public static void setTabString(Plugin plugin, Player p, int pos,
			String msg, int ping) {
		try {
			TabObject tabo = getTab(p);
			tabo.setTab(plugin, pos, msg, ping);
			playerTab.put(p.getName(), tabo);
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * Updates a players tab
	 * 
	 * A tab will be updated with the tab from the highest priority plugin
	 * 
	 * @param p
	 */
	public static void updatePlayer(Player p) {
		if (!p.isOnline())
			return;
		r = 0;
		e = 0;
		TabObject tabo = playerTab.get(p.getName());
		TabHolder tab = tabo.getTab();
		if (tab == null)
			return;

		/* need to clear the tab first */
		clearTab(p);

		for (int i = 0; i <= tabo.getLastPos(); i++) {
			// fix empty tabs
			if (tab.msgs[i] == null)
				tab.msgs[i] = nextNull();

			String msg = tab.msgs[i];
			int ping = tab.pings[i];

			addPacket(
					p,
					(msg == null) ? " " : msg.substring(0,
							Math.min(msg.length(), 16)), true, ping);
		}
		flushPackets(p, tab.getCopy());

	}

	/**
	 * Clear a players tab menu
	 * 
	 * @param p
	 */
	public static void clearTab(Player p) {
		if (!p.isOnline())
			return;

		TabHolder tabold = playerTabLast.get(p.getName());

		if (tabold != null) {
			for (String msg : tabold.msgs) {
				if (msg != null) {
					addPacket(p, msg.substring(0, Math.min(msg.length(), 16)),
							false, 0);
				}
			}
		}
	}

	public static void updateAll() {
		for (Player p : Bukkit.getOnlinePlayers()) {
			updatePlayer(p);

		}
	}

	/* return the next null filler */
	public static String nextNull() {
		String s = "";
		for (int a = 0; a < r; a++) {
			s = " " + s;
		}
		s = s + "\u00A7" + colors[e];
		e++;
		if (e > 14) {
			e = 0;
			r++;
		}
		return s;
	}
}
