package be.maximvdw.ac.utils;

import org.bukkit.entity.Player;

/**
 * @project AutismChat
 * @version 02-2014
 * @category Chat Formatting
 * @author Maxim Van de Wynckel (Maximvdw)
 * @site http://www.mvdw-software.be/
 */
public class Permissions {
	public static boolean allowOpPermissions = true; // Allow OP's to have all
	// permissions
	public static boolean denyOpPermissions = false; // Deny OP's to use any
														// permission
	public static boolean debugMode = false; // Debug mode

	/**
	 * Permission List
	 */
	public static enum Permission {
		StatusWhite, StatusRed, StatusGreen, StatusPvp, GlobalChat, Message, Reload, LoginReset
	}

	/**
	 * Check if a sender has command permissions
	 * 
	 * @param permission
	 *            Permission
	 * @param sender
	 *            Sender of the command
	 * @return Permissions true or false
	 */
	public static boolean hasPermissions(Permission permission, Object sender) {
		// Check if player
		Player player = null;
		if (!(sender instanceof Player)) {
			// Console has permissions
			return true;
		}
		player = (Player) sender; // Cast player

		// Check operator permissions
		if (player.isOp() && allowOpPermissions) {
			// Operator has permissions
			return true;
		} else if (player.isOp() && denyOpPermissions) {
			// Operator has no permissions
			return false;
		}

		// Check regular permissions
		switch (permission) {
		case StatusWhite:
			return player.hasPermission("autismchat.status.white");
		case StatusGreen:
			return player.hasPermission("autismchat.status.green");
		case StatusRed:
			return player.hasPermission("autismchat.status.red");
		case StatusPvp:
			return player.hasPermission("autismchat.toggle.pvp");
		case Message:
			return player.hasPermission("autismchat.message");
		case Reload:
			return player.hasPermission("autismchat.admin.reload");
		case GlobalChat:
			return player.hasPermission("autismchat.toggle.chat");
		case LoginReset:
			return player.hasPermission("autismchat.toggle.reset");
		default:
			return true;
		}
	}
}
